@@ -559,8 +559,18 @@ class BaseGateway(
                 self.xmpp.plugin["xep_0084"].stanza.MetaData.namespace,
                 ifrom=self.boundjid.bare,
             )
-        except (IqError, IqTimeout):
-            self.xmpp.store.users.set_avatar_hash(session.user_pk, None)
+        except IqTimeout:
+            self.log.warning("Iq timeout trying to fetch user avatar")
+            return
+        except IqError as e:
+            self.log.debug("Iq error when trying to fetch user avatar: %s", e)
+            if e.condition == "item-not-found":
+                try:
+                    await session.on_avatar(None, None, None, None, None)
+                except NotImplementedError:
+                    pass
+                else:
+                    self.xmpp.store.users.set_avatar_hash(session.user_pk, None)
             return
         await self.__dispatcher.on_avatar_metadata_info(
             session, iq["pubsub"]["items"]["item"]["avatar_metadata"]["info"]
