@@ -33,7 +33,7 @@ from ..util.types import (
     ResourceDict,
     Sticker,
 )
-from ..util.util import deprecated
+from ..util.util import deprecated, noop_coro
 
 if TYPE_CHECKING:
     from ..group.participant import LegacyParticipant
@@ -98,6 +98,7 @@ class BaseSession(
         self.ignore_messages = set[str]()
 
         self.contacts: LegacyRoster = LegacyRoster.get_self_or_unique_subclass()(self)
+        self.is_logging_in = False
         self._logged = False
         self.__reset_ready()
 
@@ -527,11 +528,13 @@ class BaseSession(
 
     @logged.setter
     def logged(self, v: bool):
+        self.is_logging_in = False
         self._logged = v
         if self.ready.done():
             if v:
                 return
             self.__reset_ready()
+            self.shutdown(logout=False)
         else:
             if v:
                 self.ready.set_result(True)
@@ -539,12 +542,15 @@ class BaseSession(
     def __repr__(self):
         return f"<Session of {self.user_jid}>"
 
-    def shutdown(self) -> asyncio.Task:
+    def shutdown(self, logout=True) -> asyncio.Task:
         for c in self.contacts:
             c.offline()
         for m in self.bookmarks:
             m.shutdown()
-        return self.xmpp.loop.create_task(self.logout())
+        if logout:
+            return self.xmpp.loop.create_task(self.logout())
+        else:
+            return self.xmpp.loop.create_task(noop_coro())
 
     @staticmethod
     def legacy_to_xmpp_msg_id(legacy_msg_id: LegacyMessageType) -> str:
@@ -657,6 +663,8 @@ class BaseSession(
             return
         for c in session.contacts:
             c.unsubscribe()
+        for m in session.bookmarks:
+            m.shutdown()
         user = cls.xmpp.store.users.get(jid)
         if user is None:
             log.warning("User not found during unregistration")
