@@ -1,6 +1,5 @@
 import logging
 import string
-import stringprep
 import uuid
 import warnings
 from copy import copy
@@ -10,9 +9,7 @@ from typing import TYPE_CHECKING, Optional, Self, Union
 
 from slixmpp import JID, InvalidJID, Message, Presence
 from slixmpp.plugins.xep_0045.stanza import MUCAdminItem
-from slixmpp.stringprep import StringprepError, resourceprep
 from slixmpp.types import MessageTypes, OptJid
-from slixmpp.util.stringprep_profiles import StringPrepError, prohibit_output
 
 from ..contact import LegacyContact
 from ..core.mixins import (
@@ -175,39 +172,22 @@ class LegacyParticipant(
         self.send_last_presence(force=True, no_cache_online=True)
 
     def __update_jid(self, unescaped_nickname: Optional[str]):
-        j: JID = copy(self.muc.jid)
-
-        if self.is_system:
-            self.jid = j
-            self._nickname_no_illegal = ""
+        if not unescaped_nickname:
+            self.jid = JID(self.muc.jid)
+            if self.is_system:
+                self._nickname_no_illegal = ""
+            else:
+                warnings.warn(
+                    "Only the system participant is allowed to not have a nickname"
+                )
+                nickname = f"unnamed-{uuid.uuid4()}"
+                self.jid.resource = self._nickname_no_illegal = nickname
             return
 
-        nickname = unescaped_nickname
-
-        if nickname:
-            nickname = self._nickname_no_illegal = strip_illegal_chars(nickname)
-        else:
-            warnings.warn(
-                "Only the system participant is allowed to not have a nickname"
-            )
-            nickname = f"unnamed-{uuid.uuid4()}"
-
-        assert isinstance(nickname, str)
-
-        try:
-            # workaround for https://codeberg.org/poezio/slixmpp/issues/3480
-            prohibit_output(nickname, [stringprep.in_table_a1])
-            resourceprep(nickname)
-        except (StringPrepError, StringprepError):
-            nickname = nickname.encode("punycode").decode()
-
-        # at this point there still might be control chars
-        try:
-            j.resource = nickname
-        except InvalidJID:
-            j.resource = strip_non_printable(nickname)
-
-        self.jid = j
+        self._nickname_no_illegal, self.jid = escape_nickname(
+            self.muc.jid,
+            unescaped_nickname,
+        )
 
     def send_configuration_change(self, codes: tuple[int]):
         if not self.is_system:
@@ -270,7 +250,7 @@ class LegacyParticipant(
                 if user_full_jid:
                     p["muc"]["jid"] = user_full_jid
                 else:
-                    jid = copy(self.user_jid)
+                    jid = JID(self.user_jid)
                     try:
                         jid.resource = next(
                             iter(self.muc.get_user_resources())  # type:ignore
@@ -539,4 +519,22 @@ class LegacyParticipant(
         return part
 
 
+def escape_nickname(muc_jid: JID, nickname: str) -> tuple[str, JID]:
+    nickname = nickname_no_illegal = strip_illegal_chars(nickname)
+
+    jid = JID(muc_jid)
+
+    try:
+        jid.resource = nickname
+    except InvalidJID:
+        nickname = nickname.encode("punycode").decode()
+        try:
+            jid.resource = nickname
+        except InvalidJID:
+            # at this point there still might be control chars
+            jid.resource = strip_non_printable(nickname)
+
+    return nickname_no_illegal, jid
+
+
 log = logging.getLogger(__name__)
