@@ -10,10 +10,12 @@ from uuid import uuid4
 
 from slixmpp import JID, Iq, Message, Presence
 from slixmpp.exceptions import IqError, IqTimeout, XMPPError
-from slixmpp.jid import _unescape_node
 from slixmpp.plugins.xep_0004 import Form
 from slixmpp.plugins.xep_0060.stanza import Item
 from slixmpp.plugins.xep_0082 import parse as str_to_datetime
+from slixmpp.plugins.xep_0469.stanza import NS as PINNING_NS
+from slixmpp.plugins.xep_0492.stanza import NS as NOTIFY_NS
+from slixmpp.plugins.xep_0492.stanza import WhenLiteral
 from slixmpp.xmlstream import ET
 
 from ..contact.contact import LegacyContact
@@ -26,8 +28,8 @@ from ..core.mixins.disco import ChatterDiscoMixin
 from ..core.mixins.lock import NamedLockMixin
 from ..core.mixins.recipient import ReactionRecipientMixin, ThreadRecipientMixin
 from ..db.models import Room
-from ..slixfix.xep_0492.stanza import WhenLiteral
 from ..util import ABCSubclassableOnceAtMost
+from ..util.jid_escaping import unescape_node
 from ..util.types import (
     HoleBound,
     LegacyGroupIdType,
@@ -40,7 +42,7 @@ from ..util.types import (
 )
 from ..util.util import deprecated, timeit, with_session
 from .archive import MessageArchive
-from .participant import LegacyParticipant
+from .participant import LegacyParticipant, escape_nickname
 
 if TYPE_CHECKING:
     from ..core.gateway import BaseGateway
@@ -182,7 +184,7 @@ class LegacyMUC(
         self.log = logging.getLogger(f"{self.user_jid}:muc:{self}")
 
     def __repr__(self):
-        return f"<MUC #{self.pk} '{self.name}' ({self.legacy_id} - {self.jid.local})'>"
+        return f"<MUC #{self.pk} '{self.name}' ({self.legacy_id} - {self.jid.user})'>"
 
     @property
     def subject_date(self) -> Optional[datetime]:
@@ -231,7 +233,7 @@ class LegacyMUC(
             self._participants_filled = True
             async for p in self.fill_participants():
                 self.__participants_store.update(p)
-                self.__store.set_participants_filled(self.pk)
+            self.__store.set_participants_filled(self.pk)
 
     async def get_participants(self) -> AsyncIterator[LegacyParticipant]:
         assert self.pk is not None
@@ -248,19 +250,21 @@ class LegacyMUC(
         async with self.lock("fill participants"):
             self._participants_filled = True
             # We only fill the participants list if/when the MUC is first
-            # joined by an XMPP client. But we may have instantiated
+            # joined by an XMPP client. But we may have instantiated some before.
             resources = set[str]()
+            async for participant in self.fill_participants():
+                # TODO: batch SQL update at the end of this function for perf?
+                self.__store_participant(participant)
+                yield participant
+                resources.add(participant.jid.resource)
             for db_participant in self.xmpp.store.participants.get_all(
                 self.pk, user_included=True
             ):
                 participant = self.Participant.from_store(
                     self.session, db_participant, muc=self
                 )
-                resources.add(participant.jid.resource)
-                yield participant
-            async for p in self.fill_participants():
-                if p.jid.resource not in resources:
-                    yield p
+                if participant.jid.resource not in resources:
+                    yield participant
             self.__store.set_participants_filled(self.pk)
             return
 
@@ -511,8 +515,7 @@ class LegacyMUC(
         return r
 
     def shutdown(self):
-        user_jid = copy(self.jid)
-        user_jid.resource = self.user_nick
+        _, user_jid = escape_nickname(self.jid, self.user_nick)
         for user_full_jid in self.user_full_jids():
             presence = self.xmpp.make_presence(
                 pfrom=user_jid, pto=user_full_jid, ptype="unavailable"
@@ -524,14 +527,13 @@ class LegacyMUC(
 
     def user_full_jids(self):
         for r in self._user_resources:
-            j = copy(self.user_jid)
+            j = JID(self.user_jid)
             j.resource = r
             yield j
 
     @property
     def user_muc_jid(self):
-        user_muc_jid = copy(self.jid)
-        user_muc_jid.resource = self.user_nick
+        _, user_muc_jid = escape_nickname(self.jid, self.user_nick)
         return user_muc_jid
 
     def _legacy_to_xmpp(self, legacy_id: LegacyMessageType):
@@ -687,6 +689,8 @@ class LegacyMUC(
         assert self.pk is not None
         p.pk = self.__participants_store.add(self.pk, p.nickname)
         self.__participants_store.update(p)
+        if p._hats:
+            self.__participants_store.set_hats(p.pk, p._hats)
 
     async def get_participant(
         self,
@@ -776,7 +780,7 @@ class LegacyMUC(
                         self.session, stored, muc=self, contact=c
                     )
 
-        nickname = c.name or _unescape_node(c.jid_username)
+        nickname = c.name or unescape_node(c.jid_username)
 
         if self.pk is None:
             nick_available = True
@@ -984,7 +988,7 @@ class LegacyMUC(
 
         :param r: The resource to kick
         """
-        pto = self.user_jid
+        pto = JID(self.user_jid)
         pto.resource = r
         p = self.xmpp.make_presence(
             pfrom=(await self.get_user_participant()).jid, pto=pto
@@ -995,6 +999,32 @@ class LegacyMUC(
         p["muc"]["status_codes"] = {110, 333}
         p.send()
 
+    async def __get_bookmark(self) -> Item | None:
+        item = Item()
+        item["id"] = self.jid
+
+        iq = Iq(stype="get", sfrom=self.user_jid, sto=self.user_jid)
+        iq["pubsub"]["items"]["node"] = self.xmpp["xep_0402"].stanza.NS
+        iq["pubsub"]["items"].append(item)
+
+        try:
+            ans = await self.xmpp["xep_0356"].send_privileged_iq(iq)
+            if len(ans["pubsub"]["items"]) != 1:
+                return None
+            # this below creates the item if it wasn't here already
+            # (slixmpp annoying magic)
+            item = ans["pubsub"]["items"]["item"]
+            item["id"] = self.jid
+            return item
+        except (IqError, IqTimeout) as exc:
+            warnings.warn(f"Cannot fetch bookmark: {exc}")
+            return None
+        except PermissionError:
+            warnings.warn(
+                "IQ privileges (XEP0356) are not set, we cannot fetch the user bookmarks"
+            )
+            return None
+
     async def add_to_bookmarks(
         self,
         auto_join=True,
@@ -1026,68 +1056,90 @@ class LegacyMUC(
             If set to ``None`` (default), the setting will be untouched. Only the "global"
             notification setting is supported (ie, per client type is not possible).
         """
-        item = Item()
-        item["id"] = self.jid
+        existing = await self.__get_bookmark() if preserve else None
 
-        iq = Iq(stype="get", sfrom=self.user_jid, sto=self.user_jid)
-        iq["pubsub"]["items"]["node"] = self.xmpp["xep_0402"].stanza.NS
-        iq["pubsub"]["items"].append(item)
+        new = Item()
+        new["id"] = self.jid
+        new["conference"]["nick"] = self.user_nick
 
-        is_update = False
-        if preserve:
-            try:
-                ans = await self.xmpp["xep_0356"].send_privileged_iq(iq)
-                is_update = len(ans["pubsub"]["items"]) == 1
-                # this below creates the item if it wasn't here already
-                # (slixmpp annoying magic)
-                item = ans["pubsub"]["items"]["item"]
-                item["id"] = self.jid
-            except (IqError, IqTimeout):
-                item["conference"]["autojoin"] = auto_join
-            except PermissionError:
-                warnings.warn(
-                    "IQ privileges (XEP0356) are not set, we cannot fetch the user bookmarks"
-                )
-            else:
-                # if the bookmark is already present, we preserve it as much as
-                # possible, especially custom <extensions>
-                self.log.debug("Existing: %s", item)
-                # if it's an update, we do not touch the auto join flag
-                if not is_update:
-                    item["conference"]["autojoin"] = auto_join
+        if existing is None:
+            change = True
+            new["conference"]["autojoin"] = auto_join
         else:
-            item["conference"]["autojoin"] = auto_join
+            change = False
+            new["conference"]["autojoin"] = existing["conference"]["autojoin"]
 
-        item["conference"]["nick"] = self.user_nick
+        existing_extensions = existing is not None and existing[
+            "conference"
+        ].get_plugin("extensions", check=True)
+
+        # preserving extensions we don't know about is a MUST
+        if existing_extensions:
+            assert existing is not None
+            for el in existing["conference"]["extensions"].xml:
+                if el.tag.startswith(f"{{{NOTIFY_NS}}}"):
+                    if notify is not None:
+                        continue
+                if el.tag.startswith(f"{{{PINNING_NS}}}"):
+                    if pin is not None:
+                        continue
+                new["conference"]["extensions"].append(el)
 
         if pin is not None:
-            item["conference"]["extensions"]["pinned"] = pin
+            if existing_extensions:
+                assert existing is not None
+                existing_pin = (
+                    existing["conference"]["extensions"].get_plugin(
+                        "pinned", check=True
+                    )
+                    is not None
+                )
+                if existing_pin != pin:
+                    change = True
+            new["conference"]["extensions"]["pinned"] = pin
 
         if notify is not None:
-            item["conference"]["extensions"]["notify"].configure(notify)
+            new["conference"]["extensions"].enable("notify")
+            if existing_extensions:
+                assert existing is not None
+                existing_notify = existing["conference"]["extensions"].get_plugin(
+                    "notify", check=True
+                )
+                if existing_notify is None:
+                    change = True
+                else:
+                    if existing_notify.get_config() != notify:
+                        change = True
+                    for el in existing_notify:
+                        new["conference"]["extensions"]["notify"].append(el)
+            new["conference"]["extensions"]["notify"].configure(notify)
 
-        iq = Iq(stype="set", sfrom=self.user_jid, sto=self.user_jid)
-        iq["pubsub"]["publish"]["node"] = self.xmpp["xep_0402"].stanza.NS
-        iq["pubsub"]["publish"].append(item)
+        if change:
+            iq = Iq(stype="set", sfrom=self.user_jid, sto=self.user_jid)
+            iq["pubsub"]["publish"]["node"] = self.xmpp["xep_0402"].stanza.NS
+            iq["pubsub"]["publish"].append(new)
 
-        iq["pubsub"]["publish_options"] = _BOOKMARKS_OPTIONS
+            iq["pubsub"]["publish_options"] = _BOOKMARKS_OPTIONS
 
-        try:
-            await self.xmpp["xep_0356"].send_privileged_iq(iq)
-        except PermissionError:
-            warnings.warn(
-                "IQ privileges (XEP0356) are not set, we cannot add bookmarks for the user"
-            )
-            # fallback by forcing invitation
-            invite = True
-        except IqError as e:
-            warnings.warn(
-                f"Something went wrong while trying to set the bookmarks: {e}"
-            )
-            # fallback by forcing invitation
-            invite = True
+            try:
+                await self.xmpp["xep_0356"].send_privileged_iq(iq)
+            except PermissionError:
+                warnings.warn(
+                    "IQ privileges (XEP0356) are not set, we cannot add bookmarks for the user"
+                )
+                # fallback by forcing invitation
+                invite = True
+            except IqError as e:
+                warnings.warn(
+                    f"Something went wrong while trying to set the bookmarks: {e}"
+                )
+                # fallback by forcing invitation
+                invite = True
+        else:
+            self.log.debug("Bookmark does not need updating.")
+            return
 
-        if invite or (config.ALWAYS_INVITE_WHEN_ADDING_BOOKMARKS and not is_update):
+        if invite or (config.ALWAYS_INVITE_WHEN_ADDING_BOOKMARKS and existing is None):
             self.session.send_gateway_invite(
                 self, reason="This group could not be added automatically for you"
             )
