@@ -218,18 +218,18 @@ class SlidgeLogger(logging.Logger):
 log = logging.getLogger(__name__)
 
 
-def get_version():
+def get_version() -> str:
     try:
         git = subprocess.check_output(
-            ["git", "rev-parse", "HEAD"], stderr=subprocess.DEVNULL
+            ["git", "rev-parse", "HEAD"],
+            stderr=subprocess.DEVNULL,
+            cwd=Path(__file__).parent,
         ).decode()
     except (FileNotFoundError, subprocess.CalledProcessError):
-        pass
+        return "NO_VERSION"
     else:
         return "git-" + git[:10]
 
-    return "NO_VERSION"
-
 
 def merge_resources(resources: dict[str, ResourceDict]) -> Optional[ResourceDict]:
     if len(resources) == 0:
